from __future__ import annotations
import dataclasses


@dataclasses.dataclass
class Security:
    api_key: str = dataclasses.field(metadata={'security': { 'scheme': True, 'type': 'apiKey', 'sub_type': 'header', 'field_name': 'x-api-key' }})
    